library benchmarks.src.static_tree.tree_benchmark;

import "package:angular2/bootstrap.dart" show bootstrap;
import "package:angular2/common.dart" show NgIf;
import "package:angular2/core.dart"
    show
        Compiler,
        Component,
        Directive,
        View,
        ViewContainerRef,
        bind,
        provide,
        Provider;
import "package:angular2/src/core/linker/dynamic_component_loader.dart"
    show ComponentRef_;
import "package:angular2/src/core/application_ref.dart" show ApplicationRef;
import "package:angular2/src/core/reflection/reflection.dart" show reflector;
import "package:angular2/src/core/reflection/reflection_capabilities.dart"
    show ReflectionCapabilities;
import "package:angular2/src/platform/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/facade/browser.dart" show window, document, gc;
import "package:angular2/src/testing/benchmark_util.dart"
    show
        getIntParameter,
        getStringParameter,
        bindAction,
        windowProfile,
        windowProfileEnd;
import "package:angular2/src/platform/browser/browser_adapter.dart"
    show BrowserDomAdapter;

List<Provider> createBindings() {
  return [];
}

setupReflector() {
  reflector.reflectionCapabilities = new ReflectionCapabilities();
}

const MAX_DEPTH = 10;
main() {
  BrowserDomAdapter.makeCurrent();
  setupReflector();
  var app;
  var appRef;
  var baselineRootTreeComponent;
  var count = 0;
  profile(create, destroy, name) {
    return () {
      windowProfile(name + " w GC");
      var duration = 0;
      var count = 0;
      while (count++ < 150) {
        gc();
        var start = window.performance.now();
        create();
        duration += window.performance.now() - start;
        destroy();
      }
      windowProfileEnd(name + " w GC");
      window.console.log(
          '''Iterations: ${ count}; time: ${ duration / count} ms / iteration''');
      windowProfile(name + " w/o GC");
      duration = 0;
      count = 0;
      while (count++ < 150) {
        var start = window.performance.now();
        create();
        duration += window.performance.now() - start;
        destroy();
      }
      windowProfileEnd(name + " w/o GC");
      window.console.log(
          '''Iterations: ${ count}; time: ${ duration / count} ms / iteration''');
    };
  }
  noop() {}
  TreeNode createData() {
    var values = count++ % 2 == 0
        ? ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "*"]
        : ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "-"];
    return buildTree(MAX_DEPTH, values, 0);
  }
  ng2DestroyDom() {
    app.initData = null;
    appRef.tick();
  }
  ng2CreateDom() {
    app.initData = createData();
    appRef.tick();
  }
  initNg2() {
    bootstrap(AppComponentWithStaticTree, createBindings()).then((ref) {
      var injector = ((ref as ComponentRef_)).injector;
      appRef = injector.get(ApplicationRef);
      app = ((ref as ComponentRef_)).hostComponent;
      bindAction("#ng2DestroyDom", ng2DestroyDom);
      bindAction("#ng2CreateDom", ng2CreateDom);
      bindAction(
          "#ng2UpdateDomProfile", profile(ng2CreateDom, noop, "ng2-update"));
      bindAction("#ng2CreateDomProfile",
          profile(ng2CreateDom, ng2DestroyDom, "ng2-create"));
    });
  }
  baselineDestroyDom() {
    baselineRootTreeComponent.update(null);
  }
  baselineCreateDom() {
    baselineRootTreeComponent.update(createData());
  }
  initBaseline() {
    var tree = DOM.createElement("tree");
    DOM.appendChild(DOM.querySelector(document, "baseline"), tree);
    baselineRootTreeComponent = new BaselineAppComponent(tree, MAX_DEPTH);
    bindAction("#baselineDestroyDom", baselineDestroyDom);
    bindAction("#baselineCreateDom", baselineCreateDom);
    bindAction("#baselineUpdateDomProfile",
        profile(baselineCreateDom, noop, "baseline-update"));
    bindAction("#baselineCreateDomProfile",
        profile(baselineCreateDom, baselineDestroyDom, "baseline-create"));
  }
  initNg2();
  initBaseline();
}

class TreeNode {
  String value;
  TreeNode left;
  TreeNode right;
  TreeNode(value, left, right) {
    this.value = value;
    this.left = left;
    this.right = right;
  }
}

buildTree(maxDepth, values, curDepth) {
  if (identical(maxDepth, curDepth)) return new TreeNode("", null, null);
  return new TreeNode(
      values[curDepth],
      buildTree(maxDepth, values, curDepth + 1),
      buildTree(maxDepth, values, curDepth + 1));
}

// http://jsperf.com/nextsibling-vs-childnodes
class BaselineAppComponent {
  var element;
  num depth;
  BaseLineTreeComponent tree = null;
  BaselineAppComponent(this.element, this.depth) {}
  update(TreeNode value) {
    if (identical(value, null)) {
      this.tree = null;
      DOM.clearNodes(this.element);
    } else {
      if (identical(this.tree, null)) {
        this.tree = new BaseLineTreeComponent(this.element, this.depth);
      }
      this.tree.update(value);
    }
  }
}

var BASELINE_TREE_TEMPLATE = null;

class BaseLineTreeComponent {
  var element;
  static getTemplate() {
    if (identical(BASELINE_TREE_TEMPLATE, null)) {
      BASELINE_TREE_TEMPLATE =
          DOM.createTemplate("<span>_<tree></tree><tree></tree></span>");
    }
    return BASELINE_TREE_TEMPLATE;
  }

  BaseLineInterpolation value;
  BaseLineTreeComponent left;
  BaseLineTreeComponent right;
  bool terminal;
  BaseLineTreeComponent(this.element, num remainingDepth) {
    var clone = DOM.firstChild(
        DOM.importIntoDoc(BaseLineTreeComponent.getTemplate().content));
    DOM.appendChild(this.element, clone);
    var child = clone.firstChild;
    this.value = new BaseLineInterpolation(child);
    this.terminal = identical(remainingDepth, 0);
    if (!this.terminal) {
      child = DOM.nextSibling(child);
      this.left = new BaseLineTreeComponent(child, remainingDepth - 1);
      child = DOM.nextSibling(child);
      this.right = new BaseLineTreeComponent(child, remainingDepth - 1);
    }
  }
  update(TreeNode value) {
    this.value.update(value.value);
    if (!this.terminal) {
      this.left.update(value.left);
      this.right.update(value.right);
    }
  }
}

class BaseLineInterpolation {
  String value;
  var textNode;
  BaseLineInterpolation(textNode) {
    this.value = null;
    this.textNode = textNode;
  }
  update(String value) {
    if (!identical(this.value, value)) {
      this.value = value;
      DOM.setText(this.textNode, value + " ");
    }
  }
}

class StaticTreeComponentBase {
  TreeNode _value;
  StaticTreeComponentBase() {
    this.data = null;
  }
  set data(TreeNode value) {
    // TODO: We need an initial value as otherwise the getter for data.value will fail

    // --> this should be already caught in change detection!
    value = !identical(value, null) ? value : new TreeNode("", null, null);
    this._value = value;
  }

  get data {
    return this._value;
  }
}

@Component(selector: "tree", inputs: const ["data"])
@View(directives: const [], template: "<span>{{data.value}} </span>")
class StaticTreeComponent0 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent0],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent1 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent1],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent2 extends StaticTreeComponentBase {
  TreeNode data;
}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent2],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent3 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent3],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent4 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent4],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent5 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent5],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent6 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent6],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent7 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent7],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent8 extends StaticTreeComponentBase {}

@Component(selector: "tree", inputs: const ["data"])
@View(
    directives: const [StaticTreeComponent8],
    template:
        '''<span> {{data.value}} <tree [data]=\'data.right\'></tree><tree [data]=\'data.left\'></tree></span>''')
class StaticTreeComponent9 extends StaticTreeComponentBase {}

@Component(selector: "app")
@View(
    directives: const [StaticTreeComponent9, NgIf],
    template: '''<tree *ngIf="initData != null" [data]=\'initData\'></tree>''')
class AppComponentWithStaticTree {
  TreeNode initData;
}
